"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.entries = entries;
exports.values = values;
exports.polyfill = polyfill;
exports.default = void 0;
var hasOwnProperty = Object.prototype.hasOwnProperty;
/**
 * Returns an array of the given object's own enumerable entries.
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/entries
 */

function entries(object) {
  // `null` and `undefined` values are not allowed.
  if (object == null) {
    throw new TypeError('Object.entries called on non-object');
  }

  var entries = [];

  for (var key in object) {
    if (hasOwnProperty.call(object, key)) {
      entries.push([key, object[key]]);
    }
  }

  return entries;
}
/**
 * Returns an array of the given object's own enumerable entries.
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/values
 */


function values(object) {
  // `null` and `undefined` values are not allowed.
  if (object == null) {
    throw new TypeError('Object.values called on non-object');
  }

  var values = [];

  for (var key in object) {
    if (hasOwnProperty.call(object, key)) {
      values.push(object[key]);
    }
  }

  return values;
}
/**
 * polyfull Object ES8
 */


function polyfill() {
  var ObjectConstructor = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : Object;

  if (!ObjectConstructor.values) {
    ObjectConstructor.values = values;
  }

  if (!ObjectConstructor.entries) {
    ObjectConstructor.entries = entries;
  }
}

exports["default"] = module.exports;
var _default = module.exports;
exports.default = _default;