"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.padEnd = padEnd;
exports.padStart = padStart;
exports.polyfill = polyfill;
exports.default = void 0;

/* eslint-disable no-extend-native, no-bitwise */
// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/padEnd
function padEnd(targetLength, padString) {
  targetLength = targetLength >> 0; // floor if number or convert non-number to 0;

  padString = String(typeof padString !== 'undefined' ? padString : ' ');

  if (this.length > targetLength) {
    return String(this);
  } else {
    targetLength = targetLength - this.length;

    if (targetLength > padString.length) {
      padString += padString.repeat(targetLength / padString.length); // append to original to ensure we are longer than needed
    }

    return String(this) + padString.slice(0, targetLength);
  }
} // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/padStart


function padStart(targetLength, padString) {
  targetLength = targetLength >> 0; // truncate if number or convert non-number to 0;

  padString = String(typeof padString !== 'undefined' ? padString : ' ');

  if (this.length > targetLength) {
    return String(this);
  } else {
    targetLength = targetLength - this.length;

    if (targetLength > padString.length) {
      padString += padString.repeat(targetLength / padString.length); // append to original to ensure we are longer than needed
    }

    return padString.slice(0, targetLength) + String(this);
  }
}
/**
 * polyfill String
 */


function polyfill() {
  var StringConstructor = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : String;

  if (!StringConstructor.prototype.padStart) {
    StringConstructor.prototype.padStart = padStart;
  }

  if (!StringConstructor.prototype.padEnd) {
    StringConstructor.prototype.padEnd = padEnd;
  }
}

exports["default"] = module.exports;
var _default = module.exports;
exports.default = _default;